require "import"
import "android.widget.*"
import "android.view.*"
import "java.net.*"
import "android.media.MediaPlayer"
import "android.view.inputmethod.InputMethodManager"
import "android.content.Context"
import "java.io.*"
import "android.os.Environment"

activity.setTitle("Google TTS")

local currentLanguage = "English"
local lastAudioUrl = nil -- Variable to store the last audio URL
local fileName = tostring(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS)) .. "/speak.tts"  -- Changed the file name

-- Function to detect if text is Arabic
function isArabic(text)
  return text:match("[%z\1-\127\194-\244][\128-\191]*"):byte() > 127
end

-- Play audio directly from the API
function textToSpeech(text)
  if text == "" then
    local msg = currentLanguage == "English" and "Please enter text to convert to speech" or "الرجاء إدخال نص لتحويله إلى صوت"
    service.speak(msg)
    return
  end

  -- Automatically detect language
  local lang = isArabic(text) and "ar" or "en"
  local encodedText = URLEncoder.encode(text, "UTF-8")
  lastAudioUrl = "https://ggtts.tiiny.io/?text=" .. encodedText .. "&lang=" .. lang -- Store the audio URL

  local mediaPlayer = MediaPlayer()
  pcall(function()
    mediaPlayer.setDataSource(lastAudioUrl)
    mediaPlayer.prepare()
    mediaPlayer.setOnCompletionListener(function()
      mediaPlayer.release()
    end)
    mediaPlayer.setOnErrorListener(function(mp, what, extra)
      local errorMsg = currentLanguage == "English" and "Failed to play the audio." or "فشل في تشغيل الصوت."
      service.speak(errorMsg)
      mp.release()
      return true
    end)
    mediaPlayer.start()
  end)
end

-- Function to download audio
function downloadAudio()
  if not lastAudioUrl then
    local msg = currentLanguage == "English" and "No audio to download. Play audio first." or "لا يوجد صوت للتنزيل. قم بتشغيل الصوت أولاً."
    service.speak(msg)
    return
  end

  local success, err = pcall(function()
    local connection = URL(lastAudioUrl).openConnection()
    connection.connect()
    local inputStream = connection.getInputStream()
    local fileOutput = FileOutputStream(fileName)  -- File will be saved as "speak.tts"
    local buffer = byte[1024]
    local len
    while true do
      len = inputStream.read(buffer)
      if len == -1 then break end
      fileOutput.write(buffer, 0, len)
    end
    inputStream.close()
    fileOutput.close()
  end)

  if success then
    local msg = currentLanguage == "English" and "Audio downloaded to Downloads folder." or "تم تنزيل الصوت في مجلد التنزيلات."
    service.speak(msg)
  else
    local msg = currentLanguage == "English" and "Error downloading the audio file." or "حدث خطأ أثناء تنزيل ملف الصوت."
    service.speak(msg)
  end
end

-- Toggle language (UI text only)
function toggleLanguage()
  if currentLanguage == "English" then
    currentLanguage = "Arabic"
    ttsInput.setHint("أدخل النص لتحويله إلى صوت...")
    playButton.setText("تشغيل الصوت")
    downloadButton.setText("تنزيل الصوت")
    languageButton.setText("التحويل إلى الإنجليزية")
    service.speak("تم تغيير اللغة إلى العربية.") -- Notify user about language change
  else
    currentLanguage = "English"
    ttsInput.setHint("Enter text to convert to speech...")
    playButton.setText("Play Audio")
    downloadButton.setText("Download Audio")
    languageButton.setText("Change to Arabic")
    service.speak("Language changed to English.") -- Notify user about language change
  end
end

-- Function to hide keyboard
function hideKeyboard()
  local inputMethodManager = activity.getSystemService(Context.INPUT_METHOD_SERVICE)
  local currentFocus = activity.getCurrentFocus()
  if currentFocus then
    inputMethodManager.hideSoftInputFromWindow(currentFocus.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS)
  end
end

-- Main layout definition
local layout = {
  LinearLayout;
  orientation = "vertical";
  layout_width = "match_parent";
  layout_height = "match_parent";
  backgroundColor = "#FFFFFF";
  {
    EditText;
    id = "ttsInput";
    hint = "Enter text to convert to speech...";  -- Translated
    layout_width = "match_parent";
    layout_height = "wrap_content";
    textColor = "#000000";
    layout_margin = "16dp";
  };
  {
    Button;
    id = "playButton";
    text = "Play Audio";  -- Translated
    textColor = "#FFFFFF";
    backgroundColor = "#008000"; -- Green color
    layout_width = "match_parent";
    layout_height = "wrap_content";
    layout_margin = "16dp";
    onClick = function()
      local ttsText = ttsInput.getText().toString():gsub("^%s*(.-)%s*$", "%1")
      textToSpeech(ttsText)
    end;
  };
  {
    Button;
    id = "downloadButton";
    text = "Download Audio";  -- Translated
    textColor = "#FFFFFF";
    backgroundColor = "#FFA500"; -- Orange color
    layout_width = "match_parent";
    layout_height = "wrap_content";
    layout_margin = "16dp";
    onClick = downloadAudio;
  };
  {
    Button;
    id = "languageButton";
    text = "Change to Arabic";  -- Translated
    textColor = "#FFFFFF";
    backgroundColor = "#0000FF"; -- Blue color
    layout_width = "match_parent";
    layout_height = "wrap_content";
    layout_margin = "16dp";
    onClick = toggleLanguage;
  };
}

-- Set the layout as the content view
activity.setContentView(loadlayout(layout))
